/*******************************************************************************
+
+  LEDA  3.1c
+
+
+  segment1.h
+
+
+  Copyright (c) 1994  by  Max-Planck-Institut fuer Informatik
+  Im Stadtwald, 6600 Saarbruecken, FRG     
+  All rights reserved.
+ 
*******************************************************************************/


#ifndef LEDA_SEGMENT1_H
#define LEDA_SEGMENT1_H

#include <LEDA/point1.h>

//------------------------------------------------------------------------------
// segment1s
//------------------------------------------------------------------------------


class segment1 {

friend class line;
friend class circle;
   
   point1 start_point;
   point1 end_point;

   double sl;
   double y_a;
   double ang;


   
public:

   GenPtr data;

 LEDA_MEMORY(segment1)
   
 segment1();                 
 segment1(const point1&, const point1&); 
 segment1(double, double, double, double) ;
 segment1(const point1&, double dir, double length);
 segment1(const segment1& s) 
 { start_point = s.start_point; 
   end_point = s.end_point; 
   sl = s.sl;
   ang = s.ang;
  }     

~segment1() {}


segment1& operator=(const segment1& s) 
 { start_point = s.start_point; 
   end_point = s.end_point; 
   sl = s.sl;
   ang = s.ang;
  return *this;
 }

operator vector()  { return vector(xcoord2()-xcoord1(), ycoord2()-ycoord1()); }

bool intersection(const segment1& s, point1& inter) const;

point start()  const      { return start_point; }
point end()    const      { return end_point; }

double xcoord1() const    { return start_point.xcoord(); } 
double xcoord2() const    { return end_point.xcoord(); }
double ycoord1() const    { return start_point.ycoord(); }
double ycoord2() const    { return end_point.ycoord(); }

double slope() const { return sl; }
double y_abs() const { return y_a;  }

double angle()     const { return ang; }
double direction() const { return ang; }

bool vertical()   const { return xcoord1() == xcoord2(); }
bool horizontal() const { return ycoord1() == ycoord2(); }
 
double length() const { return start().distance(end()); }

segment1 translate(double,double) const;
segment1 translate(const vector&) const;

double  angle(const segment1&) const;

double  distance(const segment1&) const;
double  distance(const point&) const;
double  distance() const;
double  x_proj(double) const;
double  y_proj(double) const;

segment1 rotate(const point&,double) const;
segment1 rotate(double) const;

bool  right()  const     { return xcoord1() < xcoord2(); }
bool  left()   const     { return xcoord1() > xcoord2(); }
bool  up()     const     { return ycoord1() < ycoord2(); }
bool  down()   const     { return ycoord1() > ycoord2(); }

segment1 operator+(const vector& v) const { return translate(v); }

int operator==(const segment1& s) const
{ return (start_point == s.start_point && end_point == s.end_point); }

int operator!=(const segment1& s) const { return !operator==(s);}

friend ostream& operator<<(ostream& out, const segment1& s);
friend istream& operator>>(istream& in, segment1& s);

};

inline void Print(const segment1& s, ostream& out) { out << s; } 
inline void Read(segment1& s,  istream& in)        { in >> s; }

LEDA_TYPE_PARAMETER(segment1)



template <class T> 
class _CLASSTYPE segment1_data {

public:

T& operator[](segment1 s) { return (T&)s.data; }
};




#endif
